import 'dart:async';
import 'package:flutter/material.dart';
import 'package:flutter_markdown/flutter_markdown.dart';
import 'package:super_editor/super_editor.dart';
import 'package:verttice_ui/verttice_ui.dart';

class ThemedEditor extends StatefulWidget {
  final String? markdown;
  final bool readonly;
  final Future<void> Function({required String fileContent})?
      fileContentCallback;
  const ThemedEditor({
    Key? key,
    this.markdown,
    this.readonly = false,
    this.fileContentCallback,
  }) : super(key: key);

  @override
  State<ThemedEditor> createState() => _ThemedEditorState();
}

class _ThemedEditorState extends State<ThemedEditor> {
  final _docKey = GlobalKey();
  late Document _doc;
  late DocumentEditor _docEditor;

  String _markdown = '';

  Timer? _updateTimer;
  final _markdownUpdateWaitTime = const Duration(milliseconds: 250);

  @override
  void initState() {
    super.initState();
    if (widget.markdown != null) {
      _markdown = widget.markdown!;
    }
    _doc = deserializeMarkdownToDocument(_markdown)
      ..addListener(_onDocumentChange);
    _docEditor = DocumentEditor(document: _doc as MutableDocument);

    _updateMarkdown();
  }

  void _onDocumentChange() {
    _updateTimer?.cancel();
    _updateTimer = Timer(_markdownUpdateWaitTime, _updateMarkdownAndRebuild);
  }

  void _updateMarkdownAndRebuild() {
    setState(() {
      _updateMarkdown();
    });
  }

  void _updateMarkdown() {
    _markdown = serializeDocumentToMarkdown(_doc);
  }

  @override
  void dispose() {
    _updateTimer?.cancel();
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant ThemedEditor oldWidget) {
    super.didUpdateWidget(oldWidget);
    if (widget.markdown != _markdown) {
      _markdown = widget.markdown == null ? "" : widget.markdown!;
      _doc = deserializeMarkdownToDocument(_markdown)
        ..addListener(_onDocumentChange);
      _docEditor = DocumentEditor(document: _doc as MutableDocument);

      _updateMarkdown();
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      floatingActionButton: FloatingActionButton(
        onPressed: () async {
          if (widget.fileContentCallback != null) {
            await widget.fileContentCallback!(fileContent: _markdown);
          }
        },
        child: ThemedIcons.check(),
      ),
      body: widget.readonly
          ? Markdown(
              controller: ScrollController(),
              data: _markdown,
              shrinkWrap: true,
            )
          : SuperEditor(
              key: _docKey,
              editor: _docEditor,
              scrollController: ScrollController(),
              stylesheet: defaultStylesheet.copyWith(
                documentPadding: const EdgeInsets.symmetric(
                  vertical: 56,
                  horizontal: 24,
                ),
              ),
            ),
    );
  }
}
